//
// KeyboardWindow.h
//
// Copyright (c) 2003, Roku, LLC.  All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _KEYBOARD_WINDOW_H
#define _KEYBOARD_WINDOW_H

#include <cascade/Cascade.h>

///////////////////////////////////////////////////////////////////////////////
// class KeyboardWindow


// Flags for the keybaord
#define KEYBOARD_ALPHABETICAL          0
#define KEYBOARD_QWERTY                1
#define KEYBOARD_NUMBERS               2
#define KEYBOARD_SHIFT                 4
#define KEYBOARD_SHIFT_NUMBERS         8
#define KEYBOARD_SPACE                 16
#define KEYBOARD_BACK                  32
#define KEYBOARD_DONE                  64

#define KEYBOARD_ENTRY_DONE            "Done"
#define KEYBOARD_ENTRY_BACK            "Back"

class KeyboardWindow : public CascadeWindow
{
public:
	KeyboardWindow(u32 flags = 0);
	virtual ~KeyboardWindow();
		// Use class KeyboardWindow to create an on-screen keyboard.

	void SetBackgroundBitmapFilename(CascadeString & path);
		// The keyboard window can use a standard bitmap file to render its
		// background.

	typedef void (* OnEntryProc)(const CascadeString & text, void * pClientData);
	void RegisterOnEntryCallback(OnEntryProc callback, void * pClientData);
		// Register a callback function to handle "keypresses"

	CascadeDims GetDims();
		// Use GetDims() to find the dimensions of the KeyboardWindow
		// before adding it as a child to your window.

	void Create();
		// Call Create() after sizing and positioning the KeyboardWindow.
		// This function will properly position all the letters.

	void Activate();
		// Call Activate() to activate the keyboard for input.

protected:
	virtual void PaintBackground(CascadeBitmap & bitmap);
		// Clients can provide their own background renderer by overriding
		// this function.
	
protected:
	virtual void OnPaint(CascadeBitmap & bitmap);
	virtual bool OnKeyDown(u32 nKey);
	virtual bool OnNavKey(u32 nKey, CascadeWindow * & pProposedNewFocusWnd);
	virtual bool WillAcceptFocus() { return false; }

private:
	void ToggleShift();
	void SetButtonText();
	bool AddButton(const CascadeString & text);

private:
	u32				m_flags;
	CascadeFont		m_font;
	CascadeBitmap	m_bitmapBackground;
	bool			m_bUseBackgroundBitmap;
	
	CascadeString	m_layoutstr;
	CascadeString	m_layoutstr_shift;
	u32				m_highlightX;
	u32				m_highlightY;
	u32				m_lettersPerRow;
	u32				m_nRows;
	OnEntryProc		m_callback;
	void *			m_pClientData;
	u32				m_nLetterWindows;
	bool			m_bShift;
	u32				m_nButtonsInited;
	CascadeRect		m_buttonRect;
	u32				m_stdButtonWidth;
	u32				m_stdButtonHeight;

	enum { kDefaultWidthPerChar = 28 };
	enum { kDefaultHeightPerChar = 28 };

protected:
	class LilTextWindow;
	friend class LilTextWindow;
	class LilTextWindow : public CascadeWindow
	{
	public:
		void SetText(const CascadeString & text) { m_text = text; }
		CascadeString & GetText() { return m_text; }
	protected:
		virtual void OnPaint(CascadeBitmap & bitmap);
	private:
		CascadeString m_text;
	};

private:
	LilTextWindow * m_letters;
	LilTextWindow * m_pActiveLetter;
};

///////////////////////////////////////////////////////////////////////////////
// class BGBmpKeyboardWindow

class BGBmpKeyboardWindow : public KeyboardWindow
{
public:
	BGBmpKeyboardWindow(CascadeBitmap & bitmap);
	virtual ~BGBmpKeyboardWindow();
protected:
	virtual void PaintBackground(CascadeBitmap & bitmap);
private:
	CascadeBitmap & m_bitmap;
};

#endif // _KEYBOARD_WINDOW_H



