#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <iostream>
#include <stdint.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>

#include <cascade/Cascade.h>
#include <cascade/app/CascadeInput.h> 
#include <cascade/util/CascadeFSObject.h> 
#include <cascade/util/CascadeFile.h> 
#include <cascade/widgets/CascadeSpinnerWidget.h> 

#include "fbvnc.h"
//#include "vncviewer.h"
#include "ServerSelection.h"
#include "VncPrefs.h"

extern "C" {
  extern char hostname[];
  extern int port;
  extern char* pszServer;
}

#define DELAY_OPTION_COUNT 11

#define GRID_WIDTH_OPTION_COUNT 4

#define SERVERPORT 5900

ServerSelection::ServerSelection(CascadeWindow* pMainWindow, VncSixWindow* pWindowVnc) {
  pwinMain = pMainWindow;

  uiOptionNumber = 0;
  *acOptionName = 0;

  VncPrefs* pprefs = new VncPrefs();

  // Total number of options
  iOptionsCount = 1;

  ciServers = pprefs->getServerCount();
  astrServerNames = new CascadeString[ciServers];
  for (int i = 0; i < ciServers; i++) {
    astrServerNames[i] = pprefs->getServer(i);
    cout << "server #" << i << "=" << (const char*) astrServerNames[i] << endl;
  }

  this->pWindowVnc = pWindowVnc;
}

void ServerSelection::Setup() {
  CascadeScreen screen;
  CascadeRect rect(screen.GetRect());
  SetRectAbsolute(CascadeRect(rect.x + rect.w / 2 - (429/2), 
                              (int)(rect.h * 0.85), 429, 63));

  CascadeFile normalFile;
  normalFile.SetName("/usr/local/roku/graphics/common/spinner-wide-normal.png");
  normalFile.Open("r");
  CascadeBitmap* pNormalBitmap = new CascadeBitmap;
  pNormalBitmap->CreateFromBitmapFile(normalFile);
  SetBitmapNormal(pNormalBitmap);

  CascadeFile highlightFile;
  highlightFile.SetName("/usr/local/roku/graphics/common/spinner-wide-highlight.png");
  highlightFile.Open("r");
  CascadeBitmap* pHighlightBitmap = new CascadeBitmap;
  pHighlightBitmap->CreateFromBitmapFile(highlightFile);
  SetBitmapHighlight(pHighlightBitmap);
  
  SetupCurrentOption();
}

void ServerSelection::SetupCurrentOption() {
  switch (uiOptionNumber) {
    case 0: {
      strcpy(acOptionName, "Select a Server");
      SetTextOptions(astrServerNames, ciServers);
      SetCurrentChoiceIndex(0);
    } break;
  }
}

void ServerSelection::HandleSelectEvent() {
  static char aszServer[256];
  switch (uiOptionNumber) {
    case 0: {
      cout << "Current index/server: " << GetCurrentChoiceIndex() << endl;
      //strcpy(hostname, "10.1.1.4");
      //port = 1;
      strcpy(aszServer, (const char*) astrServerNames[GetCurrentChoiceIndex()]);
      sscanf(aszServer, "%[^:]:%d", hostname, &port);
      pszServer = aszServer;
      if (port < 100)
        port += SERVERPORT;
      cout << "Selected host name=" << hostname << " port=" << port << endl;
    } break;
  }
}

bool ServerSelection::OnNavKey(u32 nKey, CascadeWindow*& pProposedNewFocusWnd) {
  switch (nKey) {
    case CK_SOUTH:	{
      if (uiOptionNumber == 0) {
        uiOptionNumber = (iOptionsCount - 1);
      } else {
        uiOptionNumber--;
      }
      SetupCurrentOption();
      RedrawScreen();
    } break;
    case CK_NORTH: {
      uiOptionNumber = (uiOptionNumber + 1) % iOptionsCount;
      SetupCurrentOption();
      RedrawScreen();
    } break;
  }

  return CascadeSpinnerWidget::OnNavKey(nKey, pProposedNewFocusWnd);
}

bool ServerSelection::OnKeyDown(u32 nKey) {
  switch (nKey) {
    case CK_EXIT: {
      pWindowVnc->OnKeyDown(CK_EXIT);
    } break;
    case CK_MENU: {
      ClearFocus();
      Vanish();
      pwinMain->SetFocus();
      RedrawScreen();
    } break;
    case CK_SELECT: {
      ClearFocus();
      Vanish();
      pwinMain->SetFocus();
      RedrawScreen();
      HandleSelectEvent();

      int iStatus = connectToVncServerRaw();
      
      if (iStatus) {
        //OnKeyDown(CK_EXIT);
      } else {
        pWindowVnc->setupVncWindow();
      }
    } break;
    default: {
      return false;
    }
  }

  return true;
}

void ServerSelection::OnPaint(CascadeBitmap & bitmap) {
  CascadeScreen screen;
  CascadeRect rect(screen.GetRect());

  CascadeColor colorYellow(253, 253, 0);

  CascadeFont font;
  CascadeFont::Attributes attr("system", CascadeFont::kNormal, 40);
  font.SetAttributes(attr);

  CascadeRect rect2;
  rect2.w = 700;
  rect2.h = 135;
  rect2.x = (rect.w - rect2.w)/ 2;
  rect2.y = rect.y + (int)(rect.h * 0.75) - 10;
  CascadeColor color(0, 0, 128, 160);
  bitmap.FillRect(rect2, color);

  CascadeDims textDims(font.GetTextExtent(acOptionName));
  CascadePoint textPoint(rect.x + rect.w / 2 - textDims.w / 2,
                         rect.y + (int)(rect.h * 0.75));
  bitmap.TextOut(textPoint, acOptionName, font, colorYellow, TEXTOUT_TOP);
  CascadeSpinnerWidget::OnPaint(bitmap);
}
