/**
 *
 * Copyright (C) 2005, Andrew Oliver
 *
 * Information about VncSix is available at: http://frequal.com/roku/VncSix/index.html
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/**
 * Stores one pref with the full list of server names, and one
 * name/value pair for each server/password.  */

#include "VncPrefs.h"

#include <iostream>

extern "C" {
#include "vncauth.h"
}

VncPrefs::VncPrefs() : CascadeSettings(CascadeString("com.frequal.VncSix")), strServerListName("ServerList"), ciServersMax(50) {
  if (!SectionExists()) {
    CreateSection();
  }

  ciServers = 0;
  readServerList();
}

void VncPrefs::readServerList() {
  CascadeString strServerList;
  ReadStringValue(strServerListName, strServerList);

  char aszList[1024];
  strcpy(aszList, (const char*) strServerList);
  char* pszServer = strtok(aszList, " ");
  ciServers = 0;
  while (pszServer != NULL) {
    astrServer[ciServers] = CascadeString(pszServer);
    //cout << "server " << ciServers << "=" << (const char*) astrServer[ciServers] << endl;
    ciServers++;
    pszServer = strtok(NULL, " ");
  }
}

void VncPrefs::writeServerList() {
  CascadeString strServerList;

  for (int i = 0; i < ciServers; i++) {
    strServerList += astrServer[i];
    //cout << "write server " << i << "=" << (const char*) astrServer[i] << endl;
    if (i < (ciServers - 1)) {
      strServerList += CascadeString(" ");
    }
  }

  WriteStringValue(strServerListName, strServerList);
}

void VncPrefs::addToList(CascadeString strServer) {
  for (int i = 0;
       i < ciServersMax;
       i++) {
    if (astrServer[i] == strServer)
      break;

    if (astrServer[i].GetLength() == 0) {
      astrServer[i] = strServer;
      ciServers++;
      break;
    }
  }

  writeServerList();
}

/** Returns 0 on successful removal, -1 if string not found */
int VncPrefs::removeFromList(CascadeString strServer) {
  int iiServer = -1;

  for (int i = 0;
       i < ciServersMax;
       i++) {
    if (astrServer[i] == strServer) {
      iiServer = i;
      break;
    }
  }

  // If we found the server to delete
  if (iiServer != -1) {
    // Move all after it one slot earlier
    for (int i = iiServer; 
         i < (ciServersMax - 1);
         i++) {
      astrServer[i] = astrServer[i+1];
    }

    writeServerList();
    return 0;
  } else { // No server found in list
    return -1; 
  }
}

int VncPrefs::getServerCount() {
  return ciServers;
}

void replaceChar(CascadeString& str, char cOrig, char cNew) {
  char sz[1024];

  strcpy(sz, (const char*) str);
  char* psz = strrchr(sz, cOrig);
  if (psz != NULL) {
    *psz = cNew;
    str = sz;
  }
}

CascadeString VncPrefs::getServer(int iiServer) {
  CascadeString strServer = astrServer[iiServer];
  replaceChar(strServer, ';', ':');

  return strServer;
}

void VncPrefs::savePassword(CascadeString strServer, const char* pszPassword) {
  char aszOrigPassword[128];
  strcpy(aszOrigPassword, pszPassword);
  char aszPassString[128];
  vncEncryptPasswdToString(aszOrigPassword, (char*) aszPassString);
  
  CascadeString strEncryptedPassword(aszPassString);

  // Fix colons
  replaceChar(strServer, ':', ';');

  WriteStringValue(strServer, strEncryptedPassword);

  // Save this server in the master list
  addToList(strServer);
}

bool VncPrefs::getPassword(CascadeString strServer, CascadeString& strPassword) {
  // Fix colons
  replaceChar(strServer, ':', ';');

  CascadeString strEncryptedPassword;
  if (ReadStringValue(strServer, strEncryptedPassword)) {
    char aszEncryptedPassString[128];
    strcpy(aszEncryptedPassString, (const char*) strEncryptedPassword);
    char aszDecryptedPassString[128];
    vncDecryptPasswdFromString(aszEncryptedPassString, aszDecryptedPassString);
  
    strPassword = aszDecryptedPassString;
    return true;
  } else {
    return false;
  }
}

int getPassword(char* pszServer, char* pszPassword) {
  VncPrefs prefs;
  CascadeString strPassword;
  if (prefs.getPassword(CascadeString(pszServer), strPassword)) {
    //cout << "server=" << pszServer << "  password=" << (const char*) strPassword << endl;
    const char* pszPasswordNew = (const char*) strPassword;
    strcpy(pszPassword, pszPasswordNew);
    return 1;
  } else {
    cout << "Unable to find password for server=" << pszServer << endl;
    return 0;
  }
}

void VncPrefs::removePassword(CascadeString strServer) {
  // Fix colons
  replaceChar(strServer, ':', ';');

  VncPrefs prefs;
  prefs.DeleteStringValue(strServer);

  if (removeFromList(strServer) != 0) {
    cout << "Unable to find server " << (const char*) strServer << " for removal." << endl;
  }
}

