#ifndef __VNCSIXWINDOW_H__
#define __VNCSIXWINDOW_H__

#include <cascade/Cascade.h>
#include <deschutes/libraries/hdmachinex225/VideoScaler.h>
#include <sys/time.h>
#include <unistd.h>
#include <map>
#include <string>

#include "KeyboardWindow.h"
#include "VncPrefs.h"

extern "C" { 
  #define CARD32 unsigned long
}

void keyHandler(const CascadeString& str, void* pData);

typedef struct Action {
  CARD32 key;
  CARD32 keyModifier;
  int mouseButton;    // 0=none, 1-3 for left, middle, right buttons 
  int mouseX;
  int mouseY;
} Action;
    

/**
 * VncSixWindow -- a simple window class that is transparent and fills
 * the screen, allowing the video pane to show through.
 */
class VncSixWindow : public CascadeWindow {
 public:
  VncSixWindow();
  virtual ~VncSixWindow();

  virtual void OnMaterialize();
  virtual void OnVanish();
  virtual void OnPaint(CascadeBitmap & bitmap);
  virtual bool OnKeyDown(u32 nKey);
  virtual bool OnNavKey(u32 nKey, CascadeWindow *&pProposedNewFocusWnd);
  virtual void VncSixWindow::OnNewText(CascadeString str);

  virtual void setupVncWindow();

  VideoScaler::ScalerColorFormat m_colorFormat;
  u8* m_pScalerVRAM;
  bool m_bStopThread;
 private:
  virtual void handleRemoteButton(u32 button);
  virtual void sendKey(CARD32 key, CARD32 keyModifier);
  virtual void moveMouse(int iXDelta, int iYDelta);
  virtual void sendMouseButtonClick(int mouseButton);
  /*
  virtual void sendPageUp();
  virtual void sendPageDown();
  virtual void sendAltLeftArrow();
  virtual void sendAltF4();
  virtual void sendLeftMouseButtonClick();
  virtual void sendMiddleMouseButtonClick();
  virtual void sendRightMouseButtonClick();
  virtual void moveNorth();
  virtual void moveSouth();
  virtual void moveWest();
  virtual void moveEast();
  */
  virtual void promptForServer();
  virtual void hideKeyboard();
  virtual void nextKeymap();
  virtual void setKeymap(std::string strKeymapName);

  bool bKeyboardVisible;
  bool bKeyboardInitialized;
  KeyboardWindow* m_pKeyboard;
  VideoScaler m_scaler;
  u32 m_nScalerBytesPerPixel;
  CascadeRect m_scalerOutputRect;
  u32 m_nScalerRowBytes;
  VncPrefs prefs;
  u32 u32MouseX;
  u32 u32MouseY;
  u32 u32MouseVelocity;
  timeval timeLastMouseMove;
  u32 u32LastMouseXDelta;
  u32 u32LastMouseYDelta;
  const u32 u32RepeatThreshold;
  const u32 u32MouseVelocityMax;
  map<u32, Action*>* pmapRemoteActions;
  map<std::string, map<u32, Action*>*> mapKeymaps;
  map<std::string, map<u32, Action*>*>::iterator iterKeymaps;
};

#endif
