/*
 *  Copyright (C) 1997, 1998 Olivetti & Oracle Research Laboratory
 *  Copyright (C) 2005 Andrew Oliver
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

/*
 * svga.c - the svgalib interface
 * Basically hacked from x.c by {ganesh,sitaram}@cse.iitb.ernet.in
 */

#include <vncviewer.h>

#include <fcntl.h>
#include <unistd.h>

#include "fbgl.h"

#define INVALID_PIXEL 0xffffffff
#define COLORMAP_SIZE 256

extern fbvnc_framebuffer_t global_framebuffer;

unsigned long BGR233ToPixel[COLORMAP_SIZE];

#if 0
static void CopyBGR233ToScreen(CARD8 *buf, int x, int y, int width,int height);
#endif

/*
 * CopyDataToScreen.
 */
void CopyDataToScreen(CARD8 *buf, int x, int y, int width, int height) {
  gl_putbox(x, y, width, height, buf);
}

#if 0
/*
 * CopyBGR233ToScreen.
 */

static void
CopyBGR233ToScreen(CARD8 *buf, int x, int y, int width, int height)
{
    int p, q;
    void *screenData = global_framebuffer.v_buf;
    CARD8 *scr8 = ((CARD8 *)screenData) + y * si.framebufferWidth + x;
    CARD16 *scr16 = ((CARD16 *)screenData) + y * si.framebufferWidth + x;
    CARD32 *scr32 = ((CARD32 *)screenData) + y * si.framebufferWidth + x;

    switch (myFormat.bitsPerPixel) {

    case 8:
	for (q = 0; q < height; q++) {
	    for (p = 0; p < width; p++) {
		*(scr8++) = BGR233ToPixel[*(buf++)];
	    }
	    scr8 += si.framebufferWidth - width;
	}
	break;

    case 16:
	for (q = 0; q < height; q++) {
	    for (p = 0; p < width; p++) {
		*(scr16++) = BGR233ToPixel[*(buf++)];
	    }
	    scr16 += si.framebufferWidth - width;
	}
	break;

    case 32:
	for (q = 0; q < height; q++) {
	    for (p = 0; p < width; p++) {
		*(scr32++) = BGR233ToPixel[*(buf++)];
	    }
	    scr32 += si.framebufferWidth - width;
	}
	break;
    }
}
#endif

void gl_setpalettecolor(int i, int r, int g, int b) {
	 printf("palette not implemented");
}

void gl_copybox(int xsrc, int ysrc, int w, int h, int x, int y) {
  int j;
  IMPORT_FRAMEBUFFER_VARS

  //printf("gl_copybox called with xsrc %d ysrc %d w %d h %d x %d y %d\n", xsrc, ysrc, w, h, x, y);

  if (NULL != v_buf) {
    if (y < ysrc) {
      Pixel *src, *dst;
      src = v_buf + ysrc * v_xsize + xsrc;
      dst = v_buf + y * v_xsize + x;
      
      for (j=0; j<h; j++) {
        memcpy(dst, src, w*sizeof(Pixel));
        src += v_xsize;
        dst += v_xsize;
      }
    } else if (y > ysrc) {
      Pixel *src, *dst;
      src = v_buf + (ysrc+h-1) * v_xsize + xsrc;
      dst = v_buf + (y+h-1) * v_xsize + x;
		
      for (j=h; j; j--) {
        memcpy(dst, src, w*sizeof(Pixel));
        src -= v_xsize;
        dst -= v_xsize;
      }
    } else /* y == ysrc */ {
      Pixel *src, *dst;
      src = v_buf + ysrc * v_xsize + xsrc;
      dst = v_buf + y * v_xsize + x;

      for (j=0; j<h; j++) {
        memmove(dst, src, w*sizeof(Pixel));
        src += v_xsize;
        dst += v_xsize;
      }
    }
  } else {
    // No video ram
    printf("ERROR: gl_copybox called while VRAM buffer pointer is NULL");
  }
}

void gl_fillbox(int x, int y, int w, int h, int col) {
  int i, j;
  IMPORT_FRAMEBUFFER_VARS
    
  //printf("gl_fillbox called with w %d h %d x %d y %d col %d\n", w, h, x, y, col);
    //col = col | 0xff000000;

  if (NULL != v_buf) {
    for (j=0; j<h; j++) {
      Pixel *buf = v_buf + (y+j) * v_xsize + x;
        
      for (i=0; i<w; i++) {
        *(buf++) = col;
      }
    }
  } else {
    // No video ram
    printf("ERROR: gl_fillbox called while VRAM buffer pointer is NULL");
  }
}

void gl_putbox(int x, int y, int w, int h, CARD8 *buf) {
  int j, k;
  int rowsize;
  Pixel *src, *dst;
  IMPORT_FRAMEBUFFER_VARS
    
  //printf("gl_putbox called with w %d h %d x %d y %d\n", w, h, x, y);
  
  if (NULL != v_buf) {
    src = (Pixel*)buf;
    dst = v_buf + y * v_xsize + x;
    rowsize = w * sizeof(Pixel);
    
    for (j=0; j<h; j++) {
      memcpy(dst, src, rowsize);
      src += w;
      dst += v_xsize;
    }
  } else {
    // No video ram
    printf("ERROR: gl_putbox called while VRAM buffer pointer is NULL");
  }
}
