/**
 * A program to write a server/password pair into the settings.
 *
 * Copyright (C) 2005, Andrew Oliver
 *
 * Information about VncSix is available at: http://frequal.com/roku/VncSix/index.html
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <iostream>

#include "VncPrefs.h"
#include "version.h"

/** Print out usage instructions on stdout */
void showUsage(char* pszProgramName) {
    cout << pszProgramName << ": Manage the list of VNC server names and passwords" << endl;
    cout << endl;
    cout << "Usage:" << endl;
    cout << "To add a new server name/password pair to the list:" << endl;
    cout << " " << pszProgramName << " server_name server_password" << endl;
    cout << "To list the stored servers and encoded passwords:" << endl;
    cout << " " << pszProgramName << " -l" << endl;
    cout << "To remove a stored server and password:" << endl;
    cout << " " << pszProgramName << " -r server_name" << endl;
}

// Save the password on the filesystem
void savePassword(CascadeString& strServer, char* pszPassword) {
  VncPrefs prefs;
  prefs.savePassword(strServer, pszPassword);
}

// Remove the password from the filesystem
void removePassword(CascadeString& strServer) {
  VncPrefs prefs;
  prefs.removePassword(strServer);
}

// Show the list of servers and encoded passwords
void showServerList() {
  VncPrefs prefs;


  int ciServers = prefs.getServerCount();

  if (ciServers > 0) {
    cout << "Stored VNC servers:" << endl;
    for (int iiServer = 0; iiServer < ciServers; iiServer++) {
      cout << " " << (iiServer + 1) << ". " << (const char*)prefs.getServer(iiServer) << endl;
    }
  } else {
    cout << "No VNC server name/password pairs are stored." << endl;
  }
}

int main(int ciArgs, char** ppszArg) {
  // Show name and version
  cout << ppszArg[0] << " version " << pszVersion << endl << endl;
  
  bool bShowServerList = false;
  bool bRemoveServer = false;
  bool bSaveServerAndPassword = false;
  CascadeString strServer;
  CascadeString strServerToRemove;
  char* pszPassword = NULL;

  // Check for the required arguments
  if (ciArgs < 2) {
    showUsage(ppszArg[0]);
    exit(1);
  }

  // Parse arguments
  int iiArg = 1;
  while (iiArg < ciArgs) {
    if ('-' == ppszArg[iiArg][0]) {
      const char cOption = ppszArg[iiArg][1];
      
      if ('l' == cOption) {
        bShowServerList = true;
      } else if ('r' == cOption) {
        // Server provided
        if ((iiArg+1) <= ciArgs) {
          bRemoveServer = true;
          strServerToRemove = CascadeString(ppszArg[iiArg+1]);
          iiArg++;
        } else { // server missing
          showUsage(ppszArg[0]);
          exit(3);
        }
      }
      
      iiArg++;
    } else {
      // Server/password provided
      if ((iiArg+2) == ciArgs) {
        bSaveServerAndPassword = true;
        strServer = CascadeString(ppszArg[iiArg]);
        pszPassword = ppszArg[iiArg+1];
        iiArg += 2;
      } else { // server and/or password missing
        showUsage(ppszArg[0]);
        exit(2);
      }
    }
  }

  if (bShowServerList) {
    showServerList();
  }

  if (bRemoveServer) {
    removePassword(strServerToRemove);

    cout << "Removed  password for server " << (const char*) strServer << endl << endl;
  }

  if (bSaveServerAndPassword) {
    savePassword(strServer, pszPassword);
    cout << "Saved password for server " << (const char*) strServer << endl << endl;
    //showServerList();
  }
  
  return 0;
}
